#include <proto.h>

#include "gtcursor.h"
#include "gtpixmap.h"
#include "gtfont.h"
#include "gterror.h"

/************************************************************************\
*
* GtCursor class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define GTCT_STD		0
#define GTCT_PIXMAP	1
#define GTCT_FONT		2

/* List of shared cursors */
static GtCursor *first=NULL;

/************************************************************************\
*
* Create and init cursor.
*
\************************************************************************/
GtCursor *GtMkCursor(GtDisplay *d,GtTag *tgs)
{
	GtCursor *obj;
	
	if(!(obj=new GtCursor()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtCursor object!\n");
		return(NULL);
	}
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init cursor.
*
\************************************************************************/
GtCursor *GtMkCursor(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtCursor *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkCursor(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Create and init shared cursor.
*
\************************************************************************/
GtCursor *GtMkSharedCursor(GtDisplay *d,GtTag *tgs)
{
	GtTag *t,*tt;
	ubyte ctype=GTCT_STD;
	GtCursor *obj,*c;
	GtDisplay *cdsp=NULL;
	GtPixmap *cshp=NULL,*cmsk=NULL;
	GtFont *cfshp=NULL,*cfmsk=NULL;
	ulong csch=0,cmch=0,cshape=XC_top_left_arrow;
	XColor cfg,cbg,*xc;
	
	cfg.pixel=0;
	cfg.flags=DoRed|DoGreen|DoBlue;
	cfg.red=0;
	cfg.green=0;
	cfg.blue=0;
	cbg.pixel=0;
	cbg.flags=DoRed|DoGreen|DoBlue;
	cbg.red=0xFFFF;
	cbg.green=0xFFFF;
	cbg.blue=0xFFFF;
	cdsp=d;
	tt=tgs;
	while((t=GtNextTag(&tt)))
	{
		switch(t->tag)
		{
			case GT_Shape:
				cshape=t->d.val;
				ctype=GTCT_STD;
				break;
			case GT_Pixmap:
				cshp=(GtPixmap *)t->d.ptr;
				cmsk=NULL;
				ctype=GTCT_PIXMAP;
				break;
			case GT_PixmapMask:
				if(ctype==GTCT_PIXMAP)
					cmsk=(GtPixmap *)t->d.ptr;
				break;
			case GT_Font:
				cfshp=(GtFont *)t->d.ptr;
				csch=0;
				cfmsk=NULL;
				ctype=GTCT_FONT;
				break;
			case GT_FontMask:
				if(ctype==GTCT_FONT)
				{
					cfmsk=(GtFont *)t->d.ptr;
					cmch=0;
				}
				break;
			case GT_ShapeChar:
				if(ctype==GTCT_FONT)
					csch=t->d.val;
				break;
			case GT_MaskChar:
				if(ctype==GTCT_FONT)
					cmch=t->d.val;
				break;
			case GT_Foreground:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(&cfg,xc,sizeof(XColor));
				break;
			case GT_Background:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(&cbg,xc,sizeof(XColor));
				break;
		}
	}
	for(c=first; c; c=c->next)
	{
		if((cdsp==c->dsp) && (ctype==c->type) &&
			(memcmp(&cfg,&c->fg,sizeof(XColor))==0) &&
			(memcmp(&cbg,&c->bg,sizeof(XColor))==0))
		{
			if((ctype==GTCT_STD) && (cshape!=c->shp.shape))
				continue;
			else if((ctype==GTCT_PIXMAP) && ((cshp!=c->shp.pix) || 
				(cmsk!=c->msk.pix)))
			{
				continue;
			}
			else if((ctype==GTCT_FONT) && ((cfshp!=c->shp.f.fnt) ||
				(cfmsk!=c->msk.f.fnt) || (csch!=c->shp.f.ch) || (cmch!=c->msk.f.ch)))
			{
				continue;
			}
			c->cnt++;
			return(c);
		}
	}
	if(!(obj=new GtCursor()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtCursor object!\n");
		return(NULL);
	}
	obj->next=first;
	first=obj;
	obj->cnt=0;
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init shared cursor.
*
\************************************************************************/
GtCursor *GtMkSharedCursor(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtCursor *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkSharedCursor(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Delete shared cursor.
*
\************************************************************************/
void GtCursor::DeInit(void)
{
	GtCursor *c;
	
	if(cnt>0)
	{
		cnt--;
		return;
	}
	if(cnt==0)
	{
		c=first;
		if(c==this)
			first=next;
		else
		{
			while(c && (c->next!=this))
				c=c->next;
			if(c)
				c->next=next;
		}
	}
	GtBase::DeInit();
}

/************************************************************************\
*
* Setup empty cursor.
*
\************************************************************************/
GtCursor::GtCursor(void)
{
	next=NULL;
	dsp=NULL;
	curs=0;
	cnt=-1;
	type=GTCT_STD;
	shp.shape=XC_top_left_arrow;
	nofreex=FALSE;
}

/************************************************************************\
*
* Destroy cursor.
*
\************************************************************************/
void GtCursor::Destroy(void)
{
	if(curs && !nofreex)
		XFreeCursor(dsp->XDsp(),curs);
	GtBase::Destroy();
}

/************************************************************************\
*
* Setup cursor.
*
\************************************************************************/
sbool GtCursor::Init(GtDisplay *d,GtTag *tgs)
{
	GtTag *t;
	XColor *xc;
	long hxp=0,hyp=0;
	
	fg.pixel=0;
	fg.flags=DoRed|DoGreen|DoBlue;
	fg.red=0;
	fg.green=0;
	fg.blue=0;
	bg.pixel=0;
	bg.flags=DoRed|DoGreen|DoBlue;
	bg.red=0xFFFF;
	bg.green=0xFFFF;
	bg.blue=0xFFFF;
	dsp=d;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Shape:
				shp.shape=t->d.val;
				type=GTCT_STD;
				break;
			case GT_Pixmap:
				shp.pix=(GtPixmap *)t->d.ptr;
				msk.pix=NULL;
				type=GTCT_PIXMAP;
				break;
			case GT_PixmapMask:
				if(type==GTCT_PIXMAP)
					msk.pix=(GtPixmap *)t->d.ptr;
				break;
			case GT_Font:
				shp.f.fnt=(GtFont *)t->d.ptr;
				shp.f.ch=0;
				msk.f.fnt=NULL;
				type=GTCT_FONT;
				break;
			case GT_FontMask:
				if(type==GTCT_FONT)
				{
					msk.f.fnt=(GtFont *)t->d.ptr;
					msk.f.ch=0;
				}
				break;
			case GT_ShapeChar:
				if(type==GTCT_FONT)
					shp.f.ch=t->d.val;
				break;
			case GT_MaskChar:
				if(type==GTCT_FONT)
					msk.f.ch=t->d.val;
				break;
			case GT_Foreground:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(&fg,xc,sizeof(XColor));
				break;
			case GT_Background:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(&bg,xc,sizeof(XColor));
				break;
			case GT_XCursor:
				curs=(Cursor)t->d.val;
				nofreex=TRUE;
				break;
			case GT_DontFreeX:
				if(t->d.val)
					nofreex=TRUE;
				else
					nofreex=FALSE;
				break;
		}
	}
	if(!dsp)
	{
		GtError(this,GTEC_NODISPLAY,"No GtDisplay!\n");
		return(FALSE);
	}
	if(curs)
		return(TRUE);
	if(type==GTCT_STD)
	{
		curs=XCreateFontCursor(dsp->XDsp(),shp.shape);
		if(curs)
			XRecolorCursor(dsp->XDsp(),curs,&fg,&bg);
	}
	else if(type==GTCT_PIXMAP)
	{
		if(!shp.pix)
		{
			GtError(this,GTEC_NOXRESOURCE,"No shape pixmap!\n");
			return(FALSE);
		}
		if(!shp.pix->IsBitmap() || (msk.pix && !msk.pix->IsBitmap()))
		{
			GtError(this,GTEC_MISC,"Not bitmaps for shape and/or mask!\n");
			return(FALSE);
		}
		shp.pix->GetV(GT_HotX,&hxp,GT_HotY,&hyp,GT_DONE);
		curs=XCreatePixmapCursor(dsp->XDsp(),shp.pix->XPix(),
			(msk.pix ? msk.pix->XPix() : None),&fg,&bg,hxp,hyp);
	}
	else if(type==GTCT_FONT)
	{
		if(!shp.f.fnt)
		{
			GtError(this,GTEC_NOXRESOURCE,"No shape font!\n");
			return(FALSE);
		}
		curs=XCreateGlyphCursor(dsp->XDsp(),shp.f.fnt->XFnt(),
			(msk.f.fnt ? msk.f.fnt->XFnt() : None),shp.f.ch,msk.f.ch,&fg,&bg);
	}
	if(!curs)
	{
		GtError(this,GTEC_NOXRESOURCE,"No X-Cursor!\n");
		return(FALSE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Find display tag and then setup normal.
*
\************************************************************************/
sbool GtCursor::Init(GtTag *tgs)
{
	GtDisplay *d;
	
	d=(GtDisplay *)GtGetTagPtr(tgs,GT_Display,NULL);
	return(this->Init(d,tgs));
}
	
/************************************************************************\
*
* Set cursor attributes.
*
\************************************************************************/
sbool GtCursor::Set(GtTag *tgs)
{
	GtTag *t;
	XColor *xc;
	sbool ch=FALSE;
	
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Foreground:
				xc=(XColor *)t->d.ptr;
				if(xc)
				{
					memcpy(&fg,xc,sizeof(XColor));
					ch=TRUE;
				}
				break;
			case GT_Background:
				xc=(XColor *)t->d.ptr;
				if(xc)
				{
					memcpy(&bg,xc,sizeof(XColor));
					ch=TRUE;
				}
				break;
		}
	}
	if(ch)
		XRecolorCursor(dsp->XDsp(),curs,&fg,&bg);
	return(TRUE);
}

/************************************************************************\
*
* Get cursor attributes.
*
\************************************************************************/
void GtCursor::Get(GtTag *tgs)
{
	GtTag *t;
	XColor *xc;
	long *ptr;
	
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Display:
				*ptr=(long)dsp;
				break;
			case GT_Shape:
				if(type==GTCT_STD)
					*ptr=shp.shape;
				break;
			case GT_Pixmap:
				if(type==GTCT_PIXMAP)
					*ptr=(long)shp.pix;
				break;
			case GT_PixmapMask:
				if(type==GTCT_PIXMAP)
					*ptr=(long)msk.pix;
				break;
			case GT_Font:
				if(type==GTCT_FONT)
					*ptr=(long)shp.f.fnt;
				break;
			case GT_FontMask:
				if(type==GTCT_FONT)
					*ptr=(long)msk.f.fnt;
				break;
			case GT_ShapeChar:
				if(type==GTCT_FONT)
					*ptr=shp.f.ch;
				break;
			case GT_MaskChar:
				if(type==GTCT_FONT)
					*ptr=msk.f.ch;
				break;
			case GT_Foreground:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(xc,&fg,sizeof(XColor));
				break;
			case GT_Background:
				xc=(XColor *)t->d.ptr;
				if(xc)
					memcpy(xc,&bg,sizeof(XColor));
				break;
			case GT_XCursor:
				*ptr=(long)curs;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtCursor::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtCursor")==0)
		return(TRUE);
	else
		return(GtBase::IsSubClassOf(cn));
}
