#ifndef INC_GTWIDGET_H
#define INC_GTWIDGET_H

#include <X11/Xlib.h>

#include "gtbase.h"
#include "gtappdisp.h"
#include "gtwindow.h"
#include "gtcolormap.h"
#include "gtgc.h"
#include "gtfont.h"
#include "gtcursor.h"
#include "gtcblist.h"

/************************************************************************\
*
* GtWidget class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Return values from Handle method */
#define GTHR_USED			0		/* For this widget and used by it */
#define GTHR_UNUSED		1		/* For this widget but not used */
#define GTHR_SEARCH		2		/* Not for this widget */

/* Object flags */
#define GTWF_MAPPED		0x0001
#define GTWF_DISABLED	0x0002
#define GTWF_NOWIN		0x0004
#define GTWF_ENTERCB	0x0008
#define GTWF_DESTROY	0x1000
#define GTWF_CHG			0x2000
#define GTWF_ONCE			0x4000
#define GTWF_UNBUILT	0x8000

/* Type of callback data for Event callback */
typedef struct GtEventCD
{
	sbool used;
	XEvent *xevent;
} GtEventCD;

/* Number of GtGC for graphics and colors */
#define GT_NR_GCS	6
#define GT_NR_COLS	6

/* Type of graphics data */
typedef struct GtGfx
{
	GtWindow *win,*shwin;
	GtColormap *cmap;
	GtFont *fnt;
	GtCursor *cur;
	GtPixmap *dpix,*bpix;
	GtGC *gcs[GT_NR_GCS];
	ulong cols[GT_NR_COLS];
	ulong bcol;
	Visual *vis;
	int depth;
	ulong pflags;
} GtGfx;

#define GTBACK	0
#define GTFORE	1
#define GTLHT		2
#define GTDRK		3
#define GTFILL	4
#define GTDIS		5

#define gtwin		(gfx->win)
#define gtfnt		(gfx->fnt)
#define gtback	(gfx->gcs[GTBACK])
#define gtfore	(gfx->gcs[GTFORE])
#define gtlht		(gfx->gcs[GTLHT])
#define gtdrk		(gfx->gcs[GTDRK])
#define gtfill	(gfx->gcs[GTFILL])
#define gtdis		(gfx->gcs[GTDIS])


/* Masks for RemapColors */
#define GTRCM_BACK		0x0001
#define GTRCM_FORE		0x0002
#define GTRCM_LIGHT		0x0004
#define GTRCM_DARK		0x0008
#define GTRCM_FILL		0x0010
#define GTRCM_DIS			0x0020
#define GTRCM_CMAP		0x0040

/* Size of disable stipple pixmap */
#define GT_DIS_SZ		16

class GtAppDisp;
class GtManager;
class GtApp;

class GtWidget : public GtBase
{
public:
		/* Standard */
	GtWidget(void);
	void Destroy(void);
	void DeInit(void);
	virtual sbool Init(char *n,GtManager *p,GtTag *tgs);
	sbool InitV(char *n,GtManager *p,ulong tag,...);
	sbool Init(GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtWidget"); }
	sbool IsSubClassOf(char *cn);
	void PrtErrHeader(void);
		/* Window management */
	virtual void Map(sbool map);
	virtual sbool Build(void);
	void Move(word xp,word yp);
	virtual void Resize(word xs,word ys);
	void MoveResize(word xp,word yp,word xs,word ys);
	virtual void Refresh(void);
		/* Events */
	virtual int Handle(XEvent *xe);
		/* Instance checks */
	sbool IsChildOf(GtWidget *o);
	sbool IsMapped(void) { return(flags & GTWF_MAPPED ? TRUE : FALSE); }
	sbool IsWindowLess(void) { return(flags & GTWF_NOWIN ? TRUE : FALSE); }
	sbool DoDestroy(void) { return(flags & GTWF_DESTROY ? TRUE : FALSE); }
	virtual sbool Manage(void) { return(TRUE); }
	virtual GtWidget *Find(char *n);
		/* Values */
	virtual void SetEMask(ulong em);
	ulong GetEMask(void) { return(emask); }
	virtual sbool SetMinMaxSize(word xs,word ys,word mxs,word mys);
	void GetMinSize(int *xs,int *ys) { *xs=minxs; *ys=minys; }
	void GetMaxSize(int *xs,int *ys) { *xs=maxxs; *ys=maxys; }
	const char *GetName(void) { return(name); }
	GtAppDisp *GetDisp(void) { return(dsp); }
	GtManager *GetParent(void) { return(parent); }
	void SetUserData(GtUDPtr ud) { userd=ud; }
	GtUDPtr GetUserData(void) { return(userd); }
	void SetLInfo(GtUDPtr li) { linfo=li; }
	GtUDPtr GetLInfo(void)		{ return(linfo); }
	GtWindow *GetWin(void) { return(gtwin); }
		/* Callbacks */
	virtual void AddCB(ulong tag,GtCallB cb,GtUDPtr ud);
	virtual void RemCB(ulong tag,GtCallB cb);
		/* Notify */
	virtual void Notify(ulong tag) { tag=tag; }
	virtual void NotifyA(ulong tag,void *ptr) { tag=tag; ptr=ptr; }
		/* Color management */
	virtual void RemapColors(uword msk,ulong bg,ulong fg,
										ulong lt,ulong dk,ulong fl,ulong ds,GtColormap *cmap);
		/* Public internal methods */
	virtual void SetPosSize(word xp,word yp,word xs,word ys);
protected:
		/* Protected internal methods */
	virtual sbool Draw(XEvent *xe) { xe=xe; return(FALSE); }
	virtual sbool Key(XEvent *xe) { xe=xe; return(FALSE); }
	virtual sbool Mouse(XEvent *xe) { xe=xe; return(FALSE); }
	virtual sbool Config(XEvent *xe) { xe=xe; return(FALSE); }
	virtual sbool Misc(XEvent *xe) { xe=xe; return(FALSE); }
	sbool PrivateGC(ulong tag);
protected:
		/* Instance variables */
	GtCBList evcb;	
	GtApp *app;
	GtAppDisp *dsp;
	GtManager *parent;
	GtGfx *gfx;
	GtUDPtr userd,linfo;
	char *name;
	ulong emask;
	word rxpos,rypos;
	word xpos,ypos,xsize,ysize;
	uword minxs,minys,maxxs,maxys;
	uword flags;
};

extern GtWidget *GtMkWidget(char *n,GtManager *p,GtTag *tgs);
extern GtWidget *GtMkWidget(char *n,GtManager *p,ulong tag,...);

#endif /* INC_GTWIDGET_H */
